@@ -20,6 +20,9 @@
  *
  */
 
+typedef struct l_settings *(*eap_tls_session_cache_load_func_t)(void);
+typedef void (*eap_tls_session_cache_sync_func_t)(const struct l_settings *);
+
 enum eap_tls_version {
 	EAP_TLS_VERSION_0               = 0x00,
 	EAP_TLS_VERSION_1               = 0x01,
@@ -48,6 +51,15 @@ void eap_tls_common_handle_request(struct eap_state *eap,
 void eap_tls_common_handle_retransmit(struct eap_state *eap,
 						const uint8_t *pkt, size_t len);
 
+struct l_certchain *eap_tls_load_client_cert(struct l_settings *settings,
+						const char *value,
+						const char *passphrase,
+						bool *out_is_encrypted);
+struct l_key *eap_tls_load_priv_key(struct l_settings *settings,
+					const char *value,
+					const char *passphrase,
+					bool *out_is_encrypted);
+
 int eap_tls_common_settings_check(struct l_settings *settings,
 						struct l_queue *secrets,
 						const char *prefix,
@@ -56,6 +68,9 @@ bool eap_tls_common_settings_load(struct eap_state *eap,
 				struct l_settings *settings, const char *prefix,
 				const struct eap_tls_variant_ops *variant_ops,
 				void *variant_data);
+void eap_tls_common_set_keys(struct eap_state *eap,
+				struct l_certchain *client_cert,
+				struct l_key *client_key);
 
 void eap_tls_common_send_empty_response(struct eap_state *eap);
 enum eap_tls_version eap_tls_common_get_negotiated_version(
@@ -69,3 +84,7 @@ bool eap_tls_common_tunnel_prf_get_bytes(struct eap_state *eap,
 void eap_tls_common_tunnel_send(struct eap_state *eap, const uint8_t *data,
 							size_t data_len);
 void eap_tls_common_tunnel_close(struct eap_state *eap);
+
+void eap_tls_set_session_cache_ops(eap_tls_session_cache_load_func_t load,
+					eap_tls_session_cache_sync_func_t sync);
+void eap_tls_forget_peer(const char *peer_id);
